unit unitARDP;
{
    Antenna Radiation Diagram Plotter
    Copyright (C) 2007  Roberto Padovani

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ComCtrls,
  ExtCtrls, StdCtrls, ColorBox, EditBtn, Spin, synhighlighterunixshellscript,
  Grids, TAGraph, DbCtrls, StrUtils, INIfiles, Math, Clipbrd, ExtDlgs;

const title = 'Antenna Radiation Diagram Plotter';
      author = 'R. Padovani';
      version = '1.0.3';
      date = '08/2007';

const ANG = 0;            // index corresponding to angles in the data vectors
      VAL = 1;            // index corresponding to values in the data vectors
      MAX_VAL = 0;        // index of maximum in the statistics vector
      MIN_VAL = 1;        // index of minimum in the statistics vector
      MAX_DIR = 2;        // index of max direction in the statistics vector
      MIN_SPAN =5;        // minimum reasonable span (dB)
      MAX_SPAN =30;       // maximum reasonable span (dB)

type

  TVector = array of array of real;
  TStats = array [0..2] of real;   // vector containing data statistics
  TPlotInfo = record
    x0,y0 : integer;                    // origin transformation
    maxx,maxy : integer;                // viewport maximums
    max: real;                          // selected maximum to calculate the scale
    scale : real;                       // scale of transformation
    span: integer;                      // detail of rapresentation
    ncurves : integer;                  // count of enable datasets
    normalized: boolean;                // normalization selected
    curve : array[1..3] of boolean;     // true if dataset is enabled for plotting
  end;
    
  { TformARDP }

  TformARDP = class(TForm)
    btnLoad2: TButton;
    btnLoad3: TButton;
    btnLoadDef1: TButton;
    btnPlot: TButton;
    btnSave: TButton;
    btnClear: TButton;
    btnClip: TButton;
    btnSaveDef: TButton;
    btnLoadDef: TButton;
    btnExec: TButton;
    btnSaveDef1: TButton;
    btnLoad1: TButton;
    chkHint: TCheckBox;
    chkFile1: TCheckBox;
    chkFile2: TCheckBox;
    chkFile3: TCheckBox;
    colorFile1: TColorBox;
    colorFile2: TColorBox;
    colorFile3: TColorBox;
    editDir2: TEdit;
    editDir3: TEdit;
    editMin1: TEdit;
    editMax1: TEdit;
    editMax2: TEdit;
    editMax3: TEdit;
    editDir1: TEdit;
    editMin2: TEdit;
    editMin3: TEdit;
    editParam1: TEdit;
    editParam2: TEdit;
    editLabel1: TEdit;
    editLabel2: TEdit;
    editLabel3: TEdit;
    editMax: TEdit;
    grid1: TStringGrid;
    grid2: TStringGrid;
    grid3: TStringGrid;
    grpStats: TGroupBox;
    grpDefault: TGroupBox;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Label32: TLabel;
    Label33: TLabel;
    lblVersion: TLabel;
    Label35: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    memoDisclaimer: TMemo;
    memoLicense: TMemo;
    openFile1: TFileNameEdit;
    grpSpan: TGroupBox;
    grpCurves: TGroupBox;
    grpExec: TGroupBox;
    grpFile1: TGroupBox;
    grpFile2: TGroupBox;
    grpFile3: TGroupBox;
    imgPlot: TImage;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    lblColor1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lblColor2: TLabel;
    lblColor3: TLabel;
    lblFile1: TLabel;
    lblFile2: TLabel;
    lblfile3: TLabel;
    openFile2: TFileNameEdit;
    openFile3: TFileNameEdit;
    openExe: TFileNameEdit;
    pages: TPageControl;
    pageFile: TTabSheet;
    pageData: TTabSheet;
    pagePlot: TTabSheet;
    radioPoly2: TRadioButton;
    radioPoly3: TRadioButton;
    radioPts1: TRadioButton;
    radioPoly1: TRadioButton;
    radioNorm: TRadioButton;
    radioMax: TRadioButton;
    grpMax: TRadioGroup;
    editSpan: TSpinEdit;
    radioPts2: TRadioButton;
    radioPts3: TRadioButton;
    savePlot: TSavePictureDialog;
    pageAbout: TTabSheet;
    procedure btnClearClick(Sender: TObject);
    procedure btnClipClick(Sender: TObject);
    procedure btnExecClick(Sender: TObject);
    procedure btnLoad1Click(Sender: TObject);
    procedure btnLoad2Click(Sender: TObject);
    procedure btnLoad3Click(Sender: TObject);
    procedure btnLoadDefClick(Sender: TObject);
    procedure btnPlotClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnSaveDefClick(Sender: TObject);
    procedure chkFile1Change(Sender: TObject);
    procedure chkFile2Change(Sender: TObject);
    procedure chkFile3Change(Sender: TObject);
    procedure chkHintChange(Sender: TObject);
    procedure colorFile1Change(Sender: TObject);
    procedure colorFile2Change(Sender: TObject);
    procedure colorFile3Change(Sender: TObject);
    procedure editLabel1Change(Sender: TObject);
    procedure editLabel2Change(Sender: TObject);
    procedure editLabel3Change(Sender: TObject);
    procedure editSpanChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pagePlotShow(Sender: TObject);
    procedure radioMaxChange(Sender: TObject);
    procedure radioNormChange(Sender: TObject);
    procedure radioPoly1Change(Sender: TObject);
    procedure radioPoly2Change(Sender: TObject);
    procedure radioPoly3Change(Sender: TObject);
    procedure radioPts1Change(Sender: TObject);
    procedure radioPts2Change(Sender: TObject);
    procedure radioPts3Change(Sender: TObject);
  private
    { private declarations }
    data : array[1..3] of TVector;      // angle and value for file 1
    numData : array[1..3] of integer;   // size of datasets
    stats : array[1..3] of TStats;      // dataset statistics
    fileNames : array [1..3] of string; // empty string if not selected
    cols: array [1..3] of TColor;       // plotting color for each data set
    labels: array [1..3] of string;     // labelling string "   "   "    "
    styles: array [1..3] of boolean;    // =true if points are connected
    iniFile : TIniFile;                 // .INI file for storing configuration
    
    plot : TBitmap;                     // bitmap for double-buffering
    info : TPlotInfo;                   // plotting details
    
    function loadFile(filename : string; grid : TStringGrid):boolean;
                                        // load filename into the correct vector
    procedure extractData(grid : TStringGrid; index : integer);
                                        // get floating point values from strings
    procedure getInfo();                // fill in info from the GUI
    procedure plotRefresh();            // draw or redraw the selected curves
    procedure plotCurve(index : integer);
                                        // trace the curve with given style
    procedure clearPlot();              // draw the background and the grid
    function getX(x : real) : integer;
    function getY(y : real) : integer;
    function radX(phi,rad: real) : real;
    function radY(phi,rad: real) : real;
  public
    { public declarations }
  end; 

var
  formARDP: TformARDP;

implementation

{ TformARDP }
procedure TformARDP.plotCurve(index : integer);// CHECK OUT plot.canvasa.interpolation !!!!
var i,x,y : integer;
    zero : real;
    vec : array of real;
begin
  SetLength(vec,numData[index]);
  zero := info.max-info.span;                       // anything below this is zeroed out
  for i := 0 to numData[index]-1 do
    if (data[index][VAL,i]<zero) then
          vec[i] := 0
      else
          vec[i] := data[index][VAL,i]-zero;

  with plot.Canvas do
  begin
    Pen.Color := TColor(cols[index]);
    Pen.Width := 2;
    //Brush.Color := TColor(cols[index]);
    //Brush.Style := bsSolid;
    
    x := getX( radX( data[index][ANG,0] , vec[0] ));
    y := getY( radY( data[index][ANG,0] , vec[0] ));
    MoveTo(x,y);
    for i := 1 to numData[index]-1 do
    begin
      x := getX( radX( data[index][ANG,i] , vec[i] ));
      y := getY( radY( data[index][ANG,i] , vec[i] ));
      if ( vec[i] > info.span ) then                    // out of scale --> bypass
        MoveTo(x,y)
      else
        if (styles[index]) then
          LineTo(x,y)
        else
          EllipseC(x,y,4,4);
    end;
    x := getX( radX( data[index][ANG,0] , vec[0] ));
    y := getY( radY( data[index][ANG,0] , vec[0] ));
    if ( vec[i] > info.span ) then // out of scale --> bypass
      MoveTo(x,y)
    else
      LineTo(x,y);
  end;
end;

procedure TformARDP.plotRefresh();
var y,magicX : integer;
    s : string;
    i : integer;
begin
  clearPlot();                                      // clean up grid
  y := 10;                                          // y offset for legend
  for i:= 1 to 3 do
    if (info.curve[i]) then
    begin
      plotCurve(i);                                 // do the real plot
      with plot.Canvas do                           // add legend
      begin
        Font.Size := 11;
        Font.Style:= [];
        Font.Color := TColor(cols[i]);
        magicX := TextWidth('Max = 000,0 @ 360');  // guess maximum text size
        s := labels[i];
        if (styles[i]) then
          s := '---  ' + s
        else
          s := '-o-  ' + s;
        TextOut(5,y,s);
        s := 'Max = ' + FloatToStrF(stats[i][MAX_VAL],ffFixed,1,1);
        s := s+ ' @ ' + FloatToStrF(stats[i][MAX_DIR],ffFixed,1,0) + '';
        // MAKE A CLEVER USE OF TextWidth !!!!
        // TextOut(plot.Width-TextWidth(s)-5,y,s);
        TextOut(plot.Width-magicX-5,y,s);
        y := y + 20;
      end;
    end;
  if (info.normalized) then                          // add title
    s := 'Normalized Plot'
  else
    s := 'Plot Scaled to '+FloatToStr(info.max);
  with plot.Canvas do
  begin
    Font.Size := 11;
    Font.Style:= [fsBold];
    Font.Color:= clBlack;
    TextOut(info.x0-TextWidth(s) div 2,info.y0-info.maxy-TextHeight(s)-5,s);
  end;
  imgPlot.Canvas.Draw(0,0,plot);                     // show it
end;

procedure TformARDP.clearPlot();
var resolution : integer;
    nrg : integer;
    i : integer;
begin
  resolution := 10;
  repeat
    resolution := resolution div 2;        // span max = 30 --> 5 fits ok
    nrg := (info.span div resolution);     // number of radial grids
  until nrg > 3;                           // should be at least 3

  with info, plot.Canvas do
  begin
    Brush.Color := clWhite;
    FillRect(0,0,imgPlot.Width,imgPlot.Height);
    Pen.Width := 1;
    Pen.Color := clSilver;

    for i:= 0 to nrg-1 do
      EllipseC(x0,y0,maxx-i*(maxx div nrg),maxy-i*(maxy div nrg));

    for i:= 0 to 23 do            // radial grid every 15 degrees
      Line(x0,y0,x0+round(radX(15*i,maxx)),y0-round(radY(15*i,maxy)));

    Font.Size := 10;
    Font.Style:= [fsBold];
    Font.Color := clBlack;
    TextOut(520,144,'45');       // degree labels every 45 degrees
    TextOut(55,144,'135');
    TextOut(64,584,'225');
    TextOut(518,581,'315');
    TextOut(x0-maxx+5,y0+5,'0');
    TextOut(x0+maxx-10,y0+5,'0');
    for i:= 1 to nrg-1 do
      TextOut(x0-maxx+i*(maxx div nrg)+5,y0+5,'-'+IntToStr(i*resolution));
    for i:= 1 to nrg-1 do
      TextOut(x0+maxx-i*(maxx div nrg)-10,y0+5,'-'+IntToStr(i*resolution));
  end;
  imgPlot.Canvas.Draw(0,0,plot);
end;

function TformARDP.getX(x : real) : integer;
begin
     with info do getX := round(x0 + x*scale);
end;

function TformARDP.getY(y : real) : integer;
begin
     with info do getY := round(y0 - y*scale);
end;

function TformARDP.radX(phi,rad: real) : real;
begin
     with info do radX := rad * cos(degtorad(phi));
end;

function TformARDP.radY(phi,rad: real) : real;
begin
     with info do radY := rad * sin(degtorad(phi));
end;

procedure TformARDP.getInfo();
var i : integer;
begin
  with info do
  begin
    curve[1] := (chkFile1.State = cbChecked);
    curve[2] := (chkFile2.State = cbChecked);
    curve[3] := (chkFile3.State = cbChecked);
    ncurves:= 0;
    for i := 1 to 3 do
        if curve[i] then ncurves := ncurves+1;

    normalized := (radioNorm.State = cbChecked);
    if (normalized) then
      begin
        max := 1;
        for i := 1 to 3 do
          if (( stats[i][MAX_VAL] > info.max ) and (curve[i])) then
            max := stats[i][MAX_VAL];
      end
    else // not normalized
      try
        max := StrToFloat(editMax.Text);
      except
        on E : Exception do
        begin
          ShowMessage('Illegal Max value! Max set to default = 100.');
          editMax.Text := '100';
          max := 100;
      end;
    end;
    if (max < 1) then
    begin
      ShowMessage('For some reason, MAX is lower than 1. This is not allowed, so MAX is set to 1');
      editMax.Text := '1';
      max := 1;
    end;
    span := editSpan.Value;
    scale := maxx / span;
  end;
end;

function TformARDP.loadFile(filename : string; grid : TStringGrid):boolean;
var
  fileRows, fileData : TStringList;
  i,j : integer;
begin
  loadFile := false;
  fileRows := TStringList.Create;                // read each line from file
  fileData := TStringList.Create;                // split lines into data
  try
    try
      fileRows.LoadFromFile(filename);               // load all lines like a list
      if (UpperCase(ExtractFileExt(filename))='.TXT') then
      begin                                          // remove Dario's header lines
         i:= 0;
         while ((i<fileRows.Count) and (LeftStr(fileRows[i],1)<>'=')) do
           i := i + 1;
         for j:= 0 to i do         // WATCH OUT HERE !! this is not an erro, it is
           fileRows.Delete(0);     // a trick to delete the rows up to the marker "="
           //              \--> always delete number 0, because the numbering is
           //                   rescaled after every deletion !!!
         fileData.Delimiter := #9;                   // TAB delimited
      end
      else
        fileData.Delimiter := #32;                  // SPACE delimited

      grid.RowCount := fileRows.Count+1;             // +1 needed for the titles !!
      for i := 0 to fileRows.Count-1 do
      begin
        fileData.DelimitedText := fileRows[i];       // automagically split
        for j := 0 to fileData.Count-1 do
          grid.Cells[j,i+1] := fileData[j];          // insert into grid_ [COL,ROW]
      end;
      loadFile := true;
    except
      on E : Exception do
      ShowMessage('File not found or invalid file format.');
    end;
  finally
    fileRows.Free;
    fileData.Free;
  end;
end;

procedure TformARDP.extractData(grid : TStringGrid; index : integer);
var i : integer;
begin
  numData[index] := grid.RowCount - 1;
  SetLength(data[index],2,numData[index]);
  for i := 0 to numData[index]-1 do
  begin
    try
       data[index][ANG,i] := StrToFloat(grid.Cells[ANG,i+1]);
    except
       on E : Exception do
       begin
         ShowMessage('Conversion error at line = '+IntToStr(i+1)+' col = '+IntToStr(ANG));
         data[index][ANG,i] := 0;
       end;
    end;
    try
       data[index][VAL,i] := StrToFloat(grid.Cells[VAL,i+1]);
    except
       on E : Exception do
       begin
         ShowMessage('Conversion error at line = '+IntToStr(i+1)+' col = '+IntToStr(VAL));
         data[index][VAL,i] := 0;
       end;
    end;
  end;
  stats[index][MAX_VAL] := data[index][VAL,0];
  stats[index][MIN_VAL] := data[index][VAL,0];
  stats[index][MAX_DIR] := data[index][ANG,0];
  for i := 1 to numData[index]-1 do
  begin
    if (data[index][VAL,i]>stats[index][MAX_VAL]) then
    begin
       stats[index][MAX_VAL] := data[index][VAL,i];
       stats[index][MAX_DIR] := data[index][ANG,i];
    end;
    if (data[index][VAL,i]<stats[index][MIN_VAL]) then
       stats[index][MIN_VAL] := data[index][VAL,i];
  end;
end;

procedure TformARDP.btnLoad1Click(Sender: TObject);
begin
  if (loadFile(openFile1.FileName,grid1)) then
  begin
       fileNames[1] := openFile1.FileName;
       extractData(grid1,1);
       editMax1.Text := FloatToStrF(stats[1][MAX_VAL],ffFixed,1,1);
       editMin1.Text := FloatToStrF(stats[1][MIN_VAL],ffFixed,1,1);
       editDir1.Text := FloatToStrF(stats[1][MAX_DIR],ffFixed,1,1);
       btnPlot.Enabled:=true;
       editMax.Enabled:=true;
       radioMax.Enabled:=true;
       radioNorm.Enabled:=true;
       editSpan.Enabled:=true;
       chkFile1.Enabled:=true;
  end;
end;

procedure TformARDP.btnExecClick(Sender: TObject);
begin
  if FileExists(openExe.FileName) then
     SysUtils.ExecuteProcess(openExe.FileName,editParam1.Text+' '+editParam2.Text)
  else
     ShowMessage('File not found.');
end;

procedure TformARDP.btnClearClick(Sender: TObject);
begin
  clearPlot();
end;

procedure TformARDP.btnClipClick(Sender: TObject);
begin
  Clipboard.Assign(imgPlot.Picture.Bitmap);
end;

procedure TformARDP.btnLoad2Click(Sender: TObject);
begin
    if (loadFile(openFile2.FileName,grid2)) then
  begin
       fileNames[2] := openFile2.FileName;
       extractData(grid2,2);
       editMax2.Text := FloatToStrF(stats[2][MAX_VAL],ffFixed,1,1);
       editMin2.Text := FloatToStrF(stats[2][MIN_VAL],ffFixed,1,1);
       editDir2.Text := FloatToStrF(stats[2][MAX_DIR],ffFixed,1,1);
       btnPlot.Enabled:=true;
       editMax.Enabled:=true;
       radioMax.Enabled:=true;
       radioNorm.Enabled:=true;
       editSpan.Enabled:=true;
       chkFile2.Enabled:=true;
  end;
end;

procedure TformARDP.btnLoad3Click(Sender: TObject);
begin
  if (loadFile(openFile3.FileName,grid3)) then
  begin
       fileNames[3] := openFile3.FileName;
       extractData(grid3,3);
       editMax3.Text := FloatToStrF(stats[3][MAX_VAL],ffFixed,1,1);
       editMin3.Text := FloatToStrF(stats[3][MIN_VAL],ffFixed,1,1);
       editDir3.Text := FloatToStrF(stats[3][MAX_DIR],ffFixed,1,1);
       btnPlot.Enabled:=true;
       editMax.Enabled:=true;
       radioMax.Enabled:=true;
       radioNorm.Enabled:=true;
       editSpan.Enabled:=true;
       chkFile3.Enabled:=true;
  end;
end;

procedure TformARDP.btnLoadDefClick(Sender: TObject);
begin
  iniFile := TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  with iniFile do
  begin
    openExe.FileName := ReadString('External','FileName','');
    editParam1.Text := ReadString('External','Param1','');
    editParam2.Text := ReadString('External','Param2','');
  end;
  iniFile.Free;
end;

procedure TformARDP.btnPlotClick(Sender: TObject);
begin
  btnSave.Enabled := true;
  btnClear.Enabled := true;
  btnClip.Enabled := true;
  getInfo();
  plotRefresh();
end;

procedure TformARDP.btnSaveClick(Sender: TObject);
var fname : string;
    i : integer;
begin
  if (info.ncurves = 1) then
  begin
    for i := 1 to 3 do
      if (info.curve[i]) then fname := fileNames[i];
    savePlot.FileName := ChangeFileExt(ExtractFileName(fname),'.bmp');
    savePlot.InitialDir := ExtractFilePath(fname);
  end;
  if (savePlot.Execute()) then
    imgPlot.Picture.SaveToFile(ChangeFileExt(savePlot.FileName,'.bmp'));
end;

procedure TformARDP.btnSaveDefClick(Sender: TObject);
begin
  iniFile := TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  with iniFile do
  begin
    WriteString('External','FileName',openExe.FileName);
    WriteString('External','Param1',editParam1.Text);
    WriteString('External','Param2',editParam2.Text);
  end;
  iniFile.Free;
end;

procedure TformARDP.chkFile1Change(Sender: TObject);
begin
  getInfo();
  plotRefresh();
end;

procedure TformARDP.chkFile2Change(Sender: TObject);
begin
  getInfo();
  plotRefresh();
end;

procedure TformARDP.chkFile3Change(Sender: TObject);
begin
  getInfo();
  plotRefresh();
end;

procedure TformARDP.chkHintChange(Sender: TObject);
var showit : boolean;
begin
    // check out the bloody hierarchy of showint !!!!!
  showit := (chkHint.State = cbChecked);
  btnSave.ShowHint := showit;
  btnClip.ShowHint := showit;
  radioNorm.ShowHint := showit;
  radioMax.ShowHint := showit;
  editMax.ShowHint := showit;
  editSpan.ShowHint := showit;
  grpCurves.ShowHint := showit;
  grpStats.ShowHint := showit;
  radioPts1.ShowHint := showit;
  radioPts2.ShowHint := showit;
  radioPts3.ShowHint := showit;
  radioPoly1.ShowHint:= showit;
  radioPoly2.ShowHint:= showit;
  radioPoly3.ShowHint:= showit;
  lblFile1.ShowHint := showit;
  lblFile1.ShowHint := showit;
  editLabel1.ShowHint := showit;
  editLabel2.ShowHint := showit;
  editLabel3.ShowHint := showit;
  openExe.ShowHint := showit;
  grpDefault.ShowHint := showit;
end;

procedure TformARDP.colorFile1Change(Sender: TObject);
begin
  cols[1] := colorFile1.Selection;
  lblColor1.Font.Color := cols[1] ;
  lblFile1.Font.Color := cols[1] ;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.colorFile2Change(Sender: TObject);
begin
  cols[2] := colorFile2.Selection;
  lblColor2.Font.Color := cols[2] ;
  lblFile2.Font.Color := cols[2] ;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.colorFile3Change(Sender: TObject);
begin
  cols[3] := colorFile3.Selection;
  lblColor3.Font.Color := cols[3] ;
  lblFile3.Font.Color := cols[3] ;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.editLabel1Change(Sender: TObject);
begin
  labels[1] := editLabel1.Text;
  lblFile1.Caption := ' '+labels[1];
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.editLabel2Change(Sender: TObject);
begin
  labels[2] := editLabel2.Text;
  lblFile2.Caption := ' '+labels[2];
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.editLabel3Change(Sender: TObject);
begin
  labels[3] := editLabel3.Text;
  lblFile3.Caption := ' '+labels[3];
  if (btnSave.Enabled) then
    plotRefresh();
end;

procedure TformARDP.editSpanChange(Sender: TObject);
begin
  getInfo();
  plotRefresh();
end;

procedure TformARDP.FormCreate(Sender: TObject);
var i : integer;
    ver : string;
begin
  formARDP.Caption := title + ' - ' + version + ' - by ' + author;
  lblVersion.Caption := 'Release ' + version + ' of ' + date;
  try
    memoLicense.Lines.LoadFromFile(ExtractFilePath(Application.ExeName)+'COPYING');
  except
    on E : Exception do
    begin
      ShowMessage('License NOT found. This is free software licensed under the GNU General Public License 2.');
      ShowMessage('Please restore the software to its original distribution. See http://sourceforge.net/projects/ardp/');
      Application.Terminate;
    end;
  end;

  iniFile := TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  ver := iniFile.ReadString('Info','Ver','');
  if (ver = '') then
     ShowMessage('Welcome to Antenna Radiation Diagram Plotter!'+Chr(13)+Chr(10)+'If you need help about the various options, please click the check-box at the top right of the interface.')
  else
     if (ver <> version) then
       ShowMessage('Version changed from ' + ver + ' to ' + version + '.');

  with iniFile,formARDP do
  begin
    Top := ReadInteger('Placement','Top', Top);
    Left := ReadInteger('Placement','Left', Left);
    openExe.FileName := ReadString('External','FileName','');
    editParam1.Text := ReadString('External','Param1','');
    editParam2.Text := ReadString('External','Param2','');
    openFile1.FileName := ReadString('Files','File1','');
    openFile2.FileName := ReadString('Files','File2','');
    openFile3.FileName := ReadString('Files','File3','');
    openExe.FileName := ReadString('Files','LastExt','');
  end;
  iniFile.Free;
  openFile1.InitialDir := ExtractFilePath(Application.ExeName);
  openFile2.InitialDir := ExtractFilePath(Application.ExeName);
  openFile3.InitialDir := ExtractFilePath(Application.ExeName);
  savePlot.InitialDir := ExtractFilePath(Application.ExeName);
  for i := 1 to 3 do
    styles[i] := true;
  labels[1] := editLabel1.Text;
  labels[2] := editLabel2.Text;
  labels[3] := editLabel3.Text;
  cols[1] := colorFile1.Selection;
  cols[2] := colorFile2.Selection;
  cols[3] := colorFile3.Selection;
  plot := TBitmap.Create;
  plot.Height := imgPlot.Height;
  plot.Width := imgPlot.Width;
  with info do
  begin
    x0 := 303;
    y0 := 368;
    maxx := 300;
    maxy := 300;
    for i := 1 to 3 do
      curve[i] := false;
    ncurves:= 0;
    normalized := false;
    max := 100;
    span:= 25;
    scale:= 0;
  end;
  clearPlot();
end;

procedure TformARDP.FormDestroy(Sender: TObject);
begin
  iniFile := TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  iniFile.WriteString('Info','Ver',version);
  iniFile.WriteInteger('Placement','Top', formARDP.Top);
  iniFile.WriteInteger('Placement','Left', formARDP.Left);
  iniFile.WriteString('Files','File1',openFile1.FileName);
  iniFile.WriteString('Files','File2',openFile2.FileName);
  iniFile.WriteString('Files','File3',openFile3.FileName);
  iniFile.WriteString('Files','LastExt',openExe.FileName);
  IniFile.Free;
  plot.Free;
end;

procedure TformARDP.pagePlotShow(Sender: TObject);
begin
  btnSave.Enabled := true;
  btnClear.Enabled:= true;
  btnClip.Enabled := true;
  // have to be dealt with better than simply this:
  // getInfo();
  // plotRefresh();
  // maybe using a flag to say if a refresh is needed !!!
end;

procedure TformARDP.radioMaxChange(Sender: TObject);
begin
  getInfo();
  plotRefresh();
end;

procedure TformARDP.radioNormChange(Sender: TObject);
begin
  getInfo();
  plotRefresh();
end;

procedure TformARDP.radioPoly1Change(Sender: TObject);
begin
  if (radioPoly1.State = cbChecked) then
     styles[1] := true;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.radioPoly2Change(Sender: TObject);
begin
  if (radioPoly2.State = cbChecked) then
     styles[2] := true;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.radioPoly3Change(Sender: TObject);
begin
  if (radioPoly3.State = cbChecked) then
     styles[3] := true;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.radioPts1Change(Sender: TObject);
begin
  if (radioPts1.State = cbChecked) then
     styles[1] := false;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.radioPts2Change(Sender: TObject);
begin
  if (radioPts2.State = cbChecked) then
     styles[2] := false;
  if (btnSave.Enabled) then
     plotRefresh();
end;

procedure TformARDP.radioPts3Change(Sender: TObject);
begin
  if (radioPts3.State = cbChecked) then
     styles[3] := false;
  if (btnSave.Enabled) then
     plotRefresh();
end;

initialization
  {$I unitARDP.lrs}

end.

